package net.minecraft.src;

/**
 * The NRTEuler class is a class that stores Euler angles. It is initially assumed that the
 * angles are in degrees, but you can set it so that it will use radians instead. Use the
 * convert method to convert degrees in radians and vice versa. Note that any conversion made
 * will actually generate a copy of the instance. This due to possible loss of information
 * during conversion.
 * 
 * @author G.A.M. Kertopermono
 *
 */
public class NRTEuler
{
	public float x;
	public float y;
	public float z;
	protected boolean inRad; 
	
	public NRTEuler()
	{
		setAngles(0, 0, 0);
		inRad = false;
	}
	
	public NRTEuler(float x, float y, float z)
	{
		this(x, y, z, false);
	}
	
	public NRTEuler(float x, float y, float z, boolean inRad)
	{
		setAngles(x, y, z);
		this.inRad = inRad;
	}
	
	public NRTEuler(Vec3D vec)
	{
		setAngles(vec);
		inRad = true;
	}
	
	public void setAngles(float x, float y, float z)
	{
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public void setAngles(Vec3D vec)
	{
		x = (float)Math.atan2(vec.yCoord, Math.sqrt(vec.xCoord * vec.xCoord + vec.zCoord * vec.zCoord));
		y = (float)Math.atan2(vec.xCoord, vec.zCoord);
		z = 0;
	}
	
	/**
	 * Converts degrees to radians, and vice versa. This returns a new NRTEuler instance,
	 * so that no information will get lost in the process, which is bound to happen due
	 * to the use of floats.
	 * @return a new NRTEuler instance containing the converted angles
	 */
	public NRTEuler convert()
	{
		float conv = NRTGlobal.NRT_DEG_TO_RAD;
		if(inRad)
		{
			conv = NRTGlobal.NRT_RAD_TO_DEG;
		}
		return new NRTEuler(x * conv, y * conv, z * conv, !inRad);
	}
	
	public NRTEuler toRad()
	{
		if(inRad)
		{
			return new NRTEuler(x, y, z, inRad);
		}
		return convert();
	}
	
	public NRTQuaternion toQuaternion()
	{
		return setQuaternion(new NRTQuaternion());
	}
	
	public NRTQuaternion setQuaternion(NRTQuaternion quat)
	{
		if(inRad)
		{
			quat.setQuaternion(convert());
		}
		quat.setQuaternion(this);
		
		return quat;
	}

	@Override
	public String toString() {
		return "NRTEuler [x=" + x + ", y=" + y + ", z=" + z + ", inRad="
				+ inRad + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (inRad ? 1231 : 1237);
		result = prime * result + Float.floatToIntBits(x);
		result = prime * result + Float.floatToIntBits(y);
		result = prime * result + Float.floatToIntBits(z);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NRTEuler other = (NRTEuler) obj;
		if (inRad != other.inRad)
			return false;
		if (Float.floatToIntBits(x) != Float.floatToIntBits(other.x))
			return false;
		if (Float.floatToIntBits(y) != Float.floatToIntBits(other.y))
			return false;
		if (Float.floatToIntBits(z) != Float.floatToIntBits(other.z))
			return false;
		return true;
	}
}
