package net.minecraft.src;

public class NRTQuaternion
{
	public float w;
	public float x;
	public float y;
	public float z;
	
	public NRTQuaternion()
	{
		setQuaternion(1, 0, 0, 0);
	}
	
	public NRTQuaternion(float w, float x, float y, float z)
	{
		setQuaternion(w, x, y, z);
	}
	
	public NRTQuaternion(NRTQuaternion quat)
	{
		setQuaternion(quat);
	}
	
	public NRTQuaternion(Vec3D vec, float angle)
	{
		setQuaternion(vec, angle);
	}
	
	public NRTQuaternion(float pitch, float yaw, float roll)
	{
		setQuaternion(pitch, yaw, roll);
	}
	
	public NRTQuaternion(NRTEuler euler)
	{
		setQuaternion(euler);
	}
	
	public void setQuaternion(float w, float x, float y, float z)
	{
		this.w = w;
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public void setQuaternion(NRTQuaternion quat)
	{
		w = quat.w;
		x = quat.x;
		y = quat.y;
		z = quat.z;
	}
	
	public void setQuaternion(Vec3D vec, float angle)
	{
		Vec3D vN = vec.normalize();
		float a = angle * 0.5F;
		
		float sA = (float)Math.sin(a);
		
		w = (float)Math.cos(a);
		x = (float)vN.xCoord * sA;
		y = (float)vN.yCoord * sA;
		z = (float)vN.zCoord * sA;
	}
	
	public void setQuaternion(float pitch, float yaw, float roll)
	{
		float piOver180 = NRTGlobal.NRT_DEG_TO_RAD;
		
		float y = yaw * piOver180 / 2.0F;
		float p = pitch * piOver180 / 2.0F;
		float r = roll * piOver180 / 2.0F;
		
		float siny = (float)Math.sin(y);
		float sinp = (float)Math.sin(p);
		float sinr = (float)Math.sin(r);
		float cosy = (float)Math.cos(y);
		float cosp = (float)Math.cos(p);
		float cosr = (float)Math.cos(r);

		this.w = cosr * cosp * cosy + sinr * sinp * siny;
		this.x = cosr * sinp * cosy + sinr * cosp * siny;
		this.y = cosr * cosp * siny - sinr * sinp * cosy;
		this.z = sinr * cosp * cosy - cosr * sinp * siny;
		
		normalize();
	}
	
	public void setQuaternion(NRTEuler euler)
	{
		euler = euler.toRad();
		setQuaternion(euler.x, euler.y, euler.z);
	}
	
	public void normalize()
	{
		float mag = w*w + x*x + y*y + z*z;
		if(mag > 0F)
		{
			mag = (float)Math.sqrt(mag);
			w/= mag;
			x/= mag;
			y/= mag;
			z/= mag;
		}
	}
	
	public void conjugate()
	{
		x = -x;
		y = -y;
		z = -z;
	}
	
	public void mul(NRTQuaternion quat)
	{
		float w = this.w;
		float x = this.x;
		float y = this.y;
		float z = this.z;
		this.w = w * quat.w - x * quat.x - y * quat.y - z * quat.z;
		this.x = w * quat.x + x * quat.w + y * quat.z - z * quat.y;
        this.y = w * quat.y - x * quat.z + y * quat.w + z * quat.x;
        this.z = w * quat.z + x * quat.y - y * quat.x + z * quat.w;
	}
	
	public NRTQuaternion getNormalized()
	{
		NRTQuaternion quat = new NRTQuaternion(this);
		quat.normalize();
		return quat;
	}
	
	public NRTQuaternion getConjugate()
	{
		NRTQuaternion quat = new NRTQuaternion(this);
		quat.conjugate();
		return quat;
	}
	
	public NRTQuaternion getMul(NRTQuaternion quat)
	{
		NRTQuaternion quat2 = new NRTQuaternion(this);
		quat2.mul(quat);
		return quat2;
	}
	
	public Vec3D rotateVector(Vec3D vec)
	{
		if(x == 0 && y == 0 && z == 0)
			return Vec3D.createVectorHelper(vec.xCoord, vec.yCoord, vec.zCoord);
		double length = vec.lengthVector();
		Vec3D vN = vec.normalize();
		
		NRTQuaternion vecQuat = new NRTQuaternion((float)vN.xCoord, (float)vN.yCoord, (float)vN.zCoord, 0.0F);
		NRTQuaternion resQuat = getMul(vecQuat.getMul(getConjugate()));
		return Vec3D.createVectorHelper(resQuat.x * length, resQuat.y * length, resQuat.z * length);
	}
	
	public NRTEuler toEuler()
	{
		return setEuler(new NRTEuler(0, 0, 0, true));
	}
	
	public NRTEuler setEuler(NRTEuler euler)
	{
		float qT = x * y + z * w;
		float roll = (float)Math.asin(2 * qT);
		float yaw;
		float pitch = 0;
		if(qT == 0.5F)
		{
			yaw = 2 * (float)Math.atan2(x, w);
		}
		else if(qT == -0.5F)
		{
			yaw = -2 * (float)Math.atan2(x, w);
		}
		else
		{
			yaw = (float)Math.atan2(2 * y * w - 2 * x * z, 1 - 2 * y * y - 2 * z * z);
			pitch = (float)Math.atan2(2 * x * w - 2 * y * z, 1 - 2 * x * x - 2 * z * z);
		}
		euler.setAngles(pitch, yaw, roll);
		return euler;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Float.floatToIntBits(w);
		result = prime * result + Float.floatToIntBits(x);
		result = prime * result + Float.floatToIntBits(y);
		result = prime * result + Float.floatToIntBits(z);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NRTQuaternion other = (NRTQuaternion) obj;
		if (Float.floatToIntBits(w) != Float.floatToIntBits(other.w))
			return false;
		if (Float.floatToIntBits(x) != Float.floatToIntBits(other.x))
			return false;
		if (Float.floatToIntBits(y) != Float.floatToIntBits(other.y))
			return false;
		if (Float.floatToIntBits(z) != Float.floatToIntBits(other.z))
			return false;
		return true;
	}

	@Override
	public String toString()
	{
		return "q = " + w + " + " + x + "i + " + y + "j + " + z + "k";
	}
}
